<?php

namespace Drupal\mfd\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\node\NodeInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldConfigInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'multilingual_form_display_widget' widget.
 *
 * @FieldWidget(
 *   id = "multilingual_form_display_widget",
 *   label = @Translation("Multilingual Form Display"),
 *   field_types = {
 *     "multilingual_form_display"
 *   },
 *   multiple_values = FALSE,
 * )
 */
class MultilingualFormDisplayWidget extends WidgetBase  {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = [];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    return $summary;
  }

  /**
   * {@inheritdoc}
   */

//  The idea here is to collect all the translation field widgets
//  set them accordingly to their language values -- as in the hook_widget_form_alter()
//  then we have this single formElement be a TREE of several elements.


  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $name = $items->getName();

//    $entity = $items->getEntity();
//    $class = get_class($this);

    $form_object = $form_state->getFormObject();
    $entity = $form_object->getEntity();

    // Language Manager and objects
    $language_manager = \Drupal::languageManager();
    $current_language = $language_manager->getCurrentLanguage()->getId();

    // Here we identify which interface we're in. Since we have to show different
    // form elements depending on the instance. Manage Fields will use FieldConfigInterface
    // and Node Edit will use NodeInterface.
    if ($entity instanceof FieldConfigInterface) {

    }


    if ($entity instanceof NodeInterface) {

      $entity_type_id = $entity->getEntityTypeId();

      $entity_type = \Drupal::service('entity.manager')->getDefinition($entity_type_id);

      $form_display = $form_state->getStorage('entity_form_display')['form_display'];

//      $region = $form_display->get('content')[$name]['region'];

      $element += [
          '#type' => 'item',
          '#tree' => TRUE,
          '#weight' => 1,
          '#description_display' => 'before',
        ];

//      TODO: We can in fact have a checbox to show or hide the Label for the section.
      //unset($element['#title']);

      $available_langcodes = ($language_manager->getLanguages());
      unset($available_langcodes[$current_language]);
      reset($available_langcodes);
      $first_language = key($available_langcodes);

      foreach ($available_langcodes as $langcode => $language) {
        $form_state->set('language',$language );

        $langcode = $language->getId();
        $language_name = $language->getName();

        if ($entity->hasTranslation($langcode)) {
          $translated_entity = $entity->getTranslation($langcode);
        } else {
          $translated_entity = $entity->addTranslation($langcode);
          $translated_entity->set('title', 'untitled');
        }

        // Create a container for the entity's fields.
        $element['value'][$langcode] = array(
          '#type' => 'details',
          '#title' => $language_name,
          '#open' => $langcode === $first_language,
        );

        foreach ($translated_entity->getFieldDefinitions() as $field_name => $definition) {
          $storage_definition = $definition->getFieldStorageDefinition();

          if (($definition->isComputed() || (!empty($storage_definition)  && $this->isFieldTranslatabilityConfigurable($entity_type, $storage_definition))) && $definition->isTranslatable()) {

            $translated_items = $translated_entity->get($field_name);
            $translated_items->filterEmptyItems();
            $translated_form = ['#parents' => []];
            $widget = $form_display->getRenderer($field_name);

            $component_form = $widget->form($translated_items, $translated_form, $form_state);
            // Now we have to do a bit of massaging to ensure namespace collision
            // in widgets doesn't happen. So, we'll modify a few values
            $field_name_with_ident = $this->getUniqueName($field_name, $langcode);
            $component_form['#field_name'] = $field_name_with_ident;
            $component_form['#multiform_display_use'] = TRUE;

            $component_form['widget']['#field_name'] = $field_name_with_ident;
            $parents_flipped = array_flip($component_form['widget']['#parents']);
            $component_form['widget']['#parents'][$parents_flipped[$field_name]] = $field_name_with_ident;

            // Create a container for the entity's fields.
            $element['value'][$langcode][$field_name] = $component_form;

          }
        }
      }
    }

    return $element;
  }

//  TODO: Add the validation and submission methods. Review from .module file.

  /**
   * Helper Methods
   */

  /**
   * Checks whether translatability should be configurable for a field.
   *
   * N.B.: Instead of module_load_include('inc', 'content_translation', 'content_translation.admin')
   * which could be changed in the future or drop altogether, we've appropriated the helper
   * function. Since this is likely not to change and we are likely to use it more often, it
   * saves memory to have it here as opposed to loading an entire external file's worth
   * of functions which would have no purpose for us.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Field\FieldStorageDefinitionInterface $definition
   *   The field storage definition.
   *
   * @return bool
   *   TRUE if field translatability can be configured, FALSE otherwise.
   *
   * @internal
   */
  protected function isFieldTranslatabilityConfigurable(EntityTypeInterface $entity_type, FieldStorageDefinitionInterface $definition) {
    // Allow to configure only fields supporting multilingual storage. We skip our
    // own fields as they are always translatable. Additionally we skip a set of
    // well-known fields implementing entity system business logic.
    return
      $definition->isTranslatable() &&
      $definition->getProvider() != 'content_translation' &&
      !in_array($definition->getName(), [$entity_type->getKey('langcode'), $entity_type->getKey('default_langcode'), 'revision_translation_affected', $this->fieldDefinition->getName()]);
  }

  /**
   * Creates a unique identifier
   *
   * @param string
   *   The field name.
   * @param string
   *   The language code.
   *
   * @return string
   *   A concatenated string between the field name and the language code.
   *
   * @internal
   */
  public function getUniqueName($field_name = 'stub', $langcode = '__') {
    return $field_name . '_' . $langcode;
  }

  /**
   * Gets the initial values for the widget.
   *
   * This is a replacement for the disabled default values functionality.
   *
   * @see address_form_field_config_edit_form_alter()
   *
   * @return array
   *   The initial values, keyed by property.
   */
  protected function getInitialValues() {
    return array();
  }

}

